"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator_helper_1 = require("@prisma/generator-helper");
const sdk_1 = require("@prisma/sdk");
const lodash_1 = __importDefault(require("lodash"));
const constants_1 = require("./constants");
const writeFileSafely_1 = require("./utils/writeFileSafely");
const { version } = require('../package.json');
function emitType(type) {
    switch (type) {
        case 'Int':
            return 'Integer';
        case 'Json':
            return 'JSON';
        case 'Bytes':
            return 'LargeBinary';
        case 'Float':
            return 'Float';
        default:
            return type;
    }
}
function emitField(model) {
    if (model.name == 'createdAt' ||
        model.name == 'created_at' ||
        model.name == 'updated_at' ||
        model.name == 'updatedAt')
        return undefined;
    if (model.relationToFields || model.relationFromFields) {
        return undefined;
    }
    const args = lodash_1.default.compact([
        emitType(model.type),
        model.isId ? 'primary_key=True' : null,
        !model.isRequired ? 'nullable=True' : null,
        model.isUnique ? 'unique=True' : null,
        model.relationToFields,
        `name="${model.name}"`,
    ]);
    return `\t${model.name} = Column(${args.join(', ')})`;
}
function emitModel(model) {
    const fields = lodash_1.default.compact(model.fields.map((f) => emitField(f))).join('\n');
    let docstring = '\t';
    if (model.documentation) {
        docstring = `\t""" ${model.documentation} """`;
    }
    const tablename = model.dbName || model.name;
    return `
class ${model.name}SQL(Base):
${docstring}
\t__tablename__ = "${tablename}"

${fields}
`;
}
function emitHeader() {
    return `
from sqlalchemy import Column, String, Integer, Boolean, DateTime, LargeBinary, Float
from sqlalchemy.dialects.postgresql import JSON
from server.utils.sql.session import Base
from sqlalchemy_utils import JSONType
`;
}
(0, generator_helper_1.generatorHandler)({
    onManifest() {
        sdk_1.logger.info(`${constants_1.GENERATOR_NAME}:Registered`);
        sdk_1.logger.info('Registered!!');
        return {
            version,
            defaultOutput: '../generated',
            prettyName: constants_1.GENERATOR_NAME,
        };
    },
    onGenerate: async (options) => {
        var _a;
        const header = emitHeader();
        const output = header +
            options.dmmf.datamodel.models
                .map((m) => emitModel(m))
                .join('\n\n')
                .replace(/\t/g, '    ');
        const writeLocation = (_a = options.generator.output) === null || _a === void 0 ? void 0 : _a.value;
        sdk_1.logger.info('Writing to ', writeLocation);
        await (0, writeFileSafely_1.writeFileSafely)(writeLocation, output);
    },
});
//# sourceMappingURL=generator.js.map